package ch.unige.solidify.controller;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.repository.SolidifyRepositoryNoSql;
import ch.unige.solidify.rest.ResourceNoSql;

public abstract class ResourceNoSqlController<T extends ResourceNoSql<T>>
        extends ResourceNoSqlReadOnlyController<T> implements ControllerWithHateoasHome {

  public ResourceNoSqlController(SolidifyRepositoryNoSql<T> itemRepository) {
    super(itemRepository);
  }

  @PostMapping
  public HttpEntity<T> create(@RequestBody T t) {
    final T item = this.itemRepository.save(t);
    return new ResponseEntity<>(item, HttpStatus.CREATED);
  }

  @DeleteMapping(SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String id) {
    final T t = this.itemRepository.findOne(this.getIndex(), id);
    if (t == null) {
      throw new SolidifyHttpErrorException(HttpStatus.NOT_FOUND, "Item " + id + " not found");
    }
    if (!this.itemRepository.delete(t)) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, "Resource " + id + " gone");
    }
    return HttpStatus.OK;
  }

  @DeleteMapping
  public HttpStatus deleteList(@RequestBody String[] ids) {
    // Remove each item
    for (final String id : ids) {
      final T t = this.itemRepository.findOne(this.getIndex(), id);
      if (t == null) {
        throw new SolidifyHttpErrorException(HttpStatus.NOT_FOUND, "Parent item " + id + " not found");
      }
      this.itemRepository.delete(t);
    }
    return HttpStatus.OK;
  }

  @PatchMapping(SolidifyConstants.URL_ID)
  public HttpEntity<T> update(@PathVariable String id, @RequestBody T t2) {
    final T t1 = this.itemRepository.findOne(this.getIndex(), id);
    if (t1 == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (t1.update(t2)) {
      final T item = this.itemRepository.update(t2);
      return new ResponseEntity<>(item, HttpStatus.OK);
    }
    return new ResponseEntity<>(t1, HttpStatus.OK);
  }

}
