package ch.unige.solidify.test.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.FileInputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.security.Security;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.test.context.junit4.SpringRunner;

import ch.unige.solidify.config.SolidifyProvider;
import ch.unige.solidify.exception.SolidifyCheckingException;
import ch.unige.solidify.model.ChecksumAlgorithm;
import ch.unige.solidify.util.ChecksumTool;

@RunWith(SpringRunner.class)
public class ChecksumTest {
  private static String CRC32 = "31ddcef0";

  private static String FILE = "banner.txt";
  private static final Logger log = LoggerFactory.getLogger(ChecksumTest.class);
  private static String MD5 = "02b6f208ab513eb07555995c33e041b3";
  private static String SHA1 = "cff06053ed173bb019693ba4987f11862f14df5e";
  private static String SHA256 = "8fa781ce5fe4e486b377c60a3a4a8e1474fe7ec026c6c58da065f0d6ce7b2689";

  @Test
  public void computeChecksumTest() {
    Security.addProvider(new SolidifyProvider());
    final String[] algoList = { "CRC32", "MD5", "SHA1", "SHA256" };
    final String[] checksumList = { CRC32, MD5, SHA1, SHA256 };
    assertEquals("Wrong checksum configuration", algoList.length, checksumList.length);
    try {
      // MessageDigest algorithm = MessageDigest.getInstance("CRC32");
      final Path path = Paths.get(new ClassPathResource(FILE).getURI());
      assertNotNull("Error in source file", path);
      final String[] checksums = ChecksumTool.computeChecksum(new FileInputStream(path.toFile()),
              algoList);
      assertEquals("Wrong checksum result", algoList.length, checksums.length);
      for (int i = 0; i < algoList.length; i++) {
        assertEquals(algoList[i], checksumList[i], checksums[i]);
      }
    } catch (final Exception e) {
      log.error("Error", e);
      assertTrue("Exception occurred", false);
    }
  }

  @Test
  public void loadChecksumEnumTest() throws SolidifyCheckingException {
    final ChecksumAlgorithm algoFromEnum = ChecksumAlgorithm.SHA256;
    final ChecksumAlgorithm algoFromString = ChecksumAlgorithm.valueOf("SHA256");
    log.info("From Enum:" + algoFromEnum.name() + " => " + algoFromEnum + " vs " + algoFromEnum);
    log.info("From String(SHA256):" + algoFromString.name() + " => " + algoFromString + " vs " + algoFromString);

    if (!algoFromEnum.equals(algoFromString)) {
      assertTrue("Objects not equal", false);
    } else {
      assertTrue("Objects equal", true);
    }
  }

}
